@@ -3,7 +3,7 @@ package App::paperback;
 use v5.10;
 use strict;
 # use warnings;
-our $VERSION = "v0.42";
+our $VERSION = "v1.00";
 
 use Exporter;
 our @ISA    = qw(Exporter);
@@ -40,7 +40,10 @@ my $IH =  $HW; # [I] US Legal Quarter (H)
 my $IW =  $FW; # [I] US Legal Quarter (W)
 
 # Page reordering and position offset schemas for "4 up":
-my @P_4_UP = (16,1,13,4,2,15,3,14,12,5,9,8,6,11,7,10);
+my @P_4UP_13PLUS = (16,1,13,4,2,15,3,14,12,5,9,8,6,11,7,10);
+my @P_4UP_9PLUS = (12,1,9,4,2,11,3,10,6,7,9999,9999,8,5);
+my @P_4UP_5PLUS = (8,1,5,4,2,7,3,6);
+my @P_4UP_1PLUS = (4,1,9999,9999,2,3);
 my @X_A6_ON_A4 = (0,$CW,$CW,$AW,0,$CW,$CW,$AW,0,$CW,$CW,$AW,0,$CW,$CW,$AW);
 my @Y_A6_ON_A4 = ($CG,$CG,$CH,$CH,$CG,$CG,$CH,$CH,$CG,$CG,$CH,$CH,$CG,$CG,$CH,$CH);
 my @X_QT_ON_LT = (0,$FW,$FW,$DW,0,$FW,$FW,$DW,0,$FW,$FW,$DW,0,$FW,$FW,$DW);
@@ -49,13 +52,16 @@ my @X_QG_ON_LG = (0,$IW,$IW,$GW,0,$IW,$IW,$GW,0,$IW,$IW,$GW,0,$IW,$IW,$GW);
 my @Y_QG_ON_LG = ($IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH,$IH);
 
 # Page reordering and position offset schemas for "2 up":
-my @P_2_UP = (16,1,15,2,14,3,13,4,12,5,11,6,10,7,9,8);
-my @Y_A5_ON_A4 = ($BG,0,$AH,$BG,$BG,0,$AH,$BG,$BG,0,$AH,$BG,$BG,0,$AH,$BG);
+my @P_2UP_13PLUS = (1,16,2,15,3,14,4,13,5,12,6,11,7,10,8,9);
+my @P_2UP_9PLUS = (1,12,2,11,3,10,4,9,5,8,6,7);
+my @P_2UP_5PLUS = (1,8,2,7,3,6,4,5);
+my @P_2UP_1PLUS = (1,4,2,3);
 my @X_A5_ON_A4 = ($BH,$BH,0,0,$BH,$BH,0,0,$BH,$BH,0,0,$BH,$BH,0,0);
-my @Y_HT_ON_LT = ($EW,0,$DH,$EW,$EW,0,$DH,$EW,$EW,0,$DH,$EW,$EW,0,$DH,$EW);
+my @Y_A5_ON_A4 = ($BG,0,$AH,$BG,$BG,0,$AH,$BG,$BG,0,$AH,$BG,$BG,0,$AH,$BG);
 my @X_HT_ON_LT = ($EH,$EH,0,0,$EH,$EH,0,0,$EH,$EH,0,0,$EH,$EH,0,0);
-my @Y_HG_ON_LG = ($HW,0,$GH,$HW,$HW,0,$GH,$HW,$HW,0,$GH,$HW,$HW,0,$GH,$HW);
+my @Y_HT_ON_LT = ($EW,0,$DH,$EW,$EW,0,$DH,$EW,$EW,0,$DH,$EW,$EW,0,$DH,$EW);
 my @X_HG_ON_LG = ($HH,$HH,0,0,$HH,$HH,0,0,$HH,$HH,0,0,$HH,$HH,0,0);
+my @Y_HG_ON_LG = ($HW,0,$GH,$HW,$HW,0,$GH,$HW,$HW,0,$GH,$HW,$HW,0,$GH,$HW);
 
 my ( $IN_FILE, $OUT_FILE );
 
@@ -95,8 +101,8 @@ ISO 216 normalised (international) page sizes:
 ANSI normalised (US) page sizes:
 
   Input page sizes Quarter Letter (4.25 x 5.5 in) and Half Letter (5.5
-  x 8,5 in) produce a Letter output page size (8.5 x 11 in). Input
-  page sizes Quarter Legal (4.25 x 7 in) and Half Legal (7 x 8,5 in)
+  x 8.5 in) produce a Letter output page size (8.5 x 11 in). Input
+  page sizes Quarter Legal (4.25 x 7 in) and Half Legal (7 x 8.5 in)
   produce a Legal output page size (8.5 x 14 in). Four Quarter-Letter
   pages will be put on each Letter page, two Half-Letter pages will be
   put on each Letter page, four Quarter-Legal pages will be put on each
@@ -119,7 +125,7 @@ END_MESSAGE
   die "[!] File '$input' is not a valid v1.4 PDF file.\n"
     if $num_pag_input == 0;
 
-  my ($rot_extra, $pagesPerSheet, @x, @y, @p);
+  my ($pagesPerSheet, @x, @y);
   for ($pgSizeInput) {
        if ($_ eq "A6") { $pagesPerSheet = 4; @x = @X_A6_ON_A4; @y = @Y_A6_ON_A4; }
     elsif ($_ eq "A5") { $pagesPerSheet = 2; @x = @X_A5_ON_A4; @y = @Y_A5_ON_A4; }
@@ -130,17 +136,27 @@ END_MESSAGE
     else {die "[!] Bad page size ($pgSizeInput). See 'paperback -h' to learn more.\n"}
   }
 
-  my $rotation;
-  my $target_page;
   my ($name) = $input =~ /(.+)\.[^.]+$/;
-  my $num_pliegos;
-
   openOutputFile("${name}-paperback.pdf");
-  $num_pliegos = $num_pag_input >> 4;
-  ($rot_extra, @p) = $pagesPerSheet == 4 ? (0, @P_4_UP) : (90, @P_2_UP);
+  my $num_pliegos = $num_pag_input >> 4;
+  my ($rot_extra, @p);
+  if ($pagesPerSheet == 4) {
+  	$rot_extra = 0;
+  	   if ($num_pag_input >= 13) { @p = @P_4UP_13PLUS; } 
+  	elsif ($num_pag_input >= 9 ) { @p = @P_4UP_9PLUS;  } 
+  	elsif ($num_pag_input >= 5 ) { @p = @P_4UP_5PLUS;  } 
+  	else                         { @p = @P_4UP_1PLUS;  }
+  } else {
+  	$rot_extra = 90;
+       if ($num_pag_input >= 13) { @p = @P_2UP_13PLUS; } 
+  	elsif ($num_pag_input >= 9 ) { @p = @P_2UP_9PLUS;  } 
+  	elsif ($num_pag_input >= 5 ) { @p = @P_2UP_5PLUS;  } 
+  	else                         { @p = @P_2UP_1PLUS;  }
+  }
+  my ($rotation, $target_page);
   for (0..$num_pliegos) {
     for (0..15) {
-      newPageInOutputFile() unless $_ % $pagesPerSheet > 0 or $numPagImposed == 0;
+      newPageInOutputFile() if $_ % $pagesPerSheet == 0;
       $target_page = $p[$_] + ($numPagImposed >> 4) * 16;
       next if $target_page > $num_pag_input;
 
@@ -789,7 +805,6 @@ sub openOutputFile {
   $GobjNr      = 2;  # Objeto reservado 1 para raíz y 2 para nodo lateral inicial
   $Gparents[0] = 2;
 
-  newPageInOutputFile();
   setInitGrState();
   return;
 }
